#include <Windows.h>
#include <stdio.h>
#include <stdint.h>
#include "Ft602.h"
#include "getopt.h"



HANDLE hNotification;


struct i2c_access {
	uint8_t addr;
	uint8_t len;
	uint16_t reg;

	bool reg_is_16_bit; /* 16bit register */
	bool read_access; /* false means write access */
};

typedef struct FRAME_CFG_
{
	BYTE bStatus;
	BYTE FPS;
	WORD wWidth;
	WORD wHeight;
}FRAME_CFG, *PFRAME_CFG;

static VOID NotificationCallback(PVOID pvCallbackContext, E_FT_NOTIFICATION_CALLBACK_TYPE eCallbackType, PVOID pvCallbackInfo)
{
	/* signal the event */
	SetEvent(hNotification);
	printf("Notification callback received : %d\n", eCallbackType);
}

#define UVC_SLAVE_ADDRESS 0x0D
#define GET_UVC_REG_INDEX(channel, reg) (0x40 + 0x30 * (channel) + (reg))

static inline FT_STATUS FT_I2CAccess(FT_HANDLE ftHandle,
	const struct i2c_access* i2c, uint8_t* buf)
{
	FT_SETUP_PACKET fsp = { 0 };
	ULONG ret;

	if (!i2c || !buf)
		return FT_INVALID_PARAMETER;
	if (i2c->len > 128)
		return FT_INSUFFICIENT_RESOURCES;
	fsp.RequestType = (2 << 5);
	fsp.Request = 0x5;
	fsp.Value = *((uint16_t*)i2c + 1);
	fsp.Index = *(uint16_t*)i2c;


	if (i2c->read_access)
		fsp.RequestType |= 0x80;

	if (i2c->reg_is_16_bit)
		fsp.Value |= 0x80;
	else
		fsp.Value &= ~0x80;
	fsp.Length = i2c->len;
	return FT_ControlTransfer(ftHandle, fsp, buf, i2c->len, &ret);
}

static inline FT_STATUS FT_UVCRegAccess(FT_HANDLE ftHandle, uint8_t channel,
	uint8_t reg, uint16_t len, bool read_access, uint8_t* buf)
{
	const struct i2c_access ia = {
		UVC_SLAVE_ADDRESS,
		len,
		GET_UVC_REG_INDEX(channel, reg),
		false,
		read_access
	};

	return FT_I2CAccess(ftHandle, &ia, buf);
}



void display_usage(char* prog)
{
	printf("\nusage:\n    %s -a i2cAddr -o offset <options> \n", prog);
	printf("    %s -i \n",prog);

	printf("       : <options> \n");
	printf("       :         -r (for read) \n");
	printf("       :         -w (for write) \n");
	printf("       :         -l (size to read/write) \n");
	printf("       :         -i (notification on interrupt (IRQ line)) \n");

}

static struct option long_options[] = {
	{ "addr", required_argument, 0, 'a' },
	{ "offset", required_argument, 0, 'o' },
	{ "read", no_argument, 0, 'r' },
	{ "write", no_argument, 0, 'w' },
	{ "help", no_argument, 0, 'h' },
	{ "length", required_argument, 0, 'l' },
	{ "irq", no_argument, 0, 'i'},
	{ 0, 0, 0, 0 }
};


int main(int argc, char* argv[])
{
	FT_HANDLE ftHandle;
	FT_STATUS ftStatus;
	bool bWrite = FALSE, bIrq = FALSE;
	int i2caddr = 0, val = 0, len = 2;
	int reg = 0;
	int rdMem[4];	
	int long_index = 0;
	int opt;

	if(argc < 2)
	{
		display_usage(argv[0]);
		return 0;
	}

	while ((opt = getopt_long(argc, argv, "hriw:a:o:l:",
		long_options, &long_index)) != -1) {

		switch (opt)
		{
			default:
			case 'h':
				display_usage(argv[0]);
				return 0;
			case 'l' : len = strtoul(optarg, NULL, 0);
				break;
			case 'a': i2caddr = strtoul(optarg, NULL, 0);
				break;
			case 'o': reg = strtoul(optarg, NULL, 0);
				break;
			case 'r': bWrite = FALSE; 
				break;
			case 'w': bWrite = TRUE;
				rdMem[0] = strtoul(optarg, NULL, 0);
				break;
			case 'i': bIrq = TRUE;
				break;
			
		}
	}
			
			
	ftStatus = FT_Create(0, FT_OPEN_BY_INDEX, &ftHandle);
	if (FT_FAILED(ftStatus))
	{
		printf("FT_Create failed %d\n", ftStatus);
		goto exit;
	}

	if (bIrq) /* IRQ notification*/
	{
		/* global event */
		hNotification = CreateEvent(NULL, TRUE, FALSE, NULL);
		if (!hNotification)
		{
			goto exit;
		}
		
		/* set a callback function */
		FT_SetNotificationCallback(ftHandle, NotificationCallback, NULL);
		
		/* wait for the event to signal*/
		DWORD dwRet = WaitForSingleObject(hNotification, INFINITE);
		
	}
	else
	{
		struct i2c_access i2c;

		/* initialize the i2c strutcure */
		i2c.addr = i2caddr;
		i2c.len = len;
		i2c.read_access = !bWrite;
		i2c.reg = reg;
		i2c.reg_is_16_bit = 0;


		FT_I2CAccess(ftHandle, &i2c, (uint8_t*)&rdMem[0]);
		if (!bWrite)
		{
			int i = 0;
			UCHAR *buff = (UCHAR *)&rdMem[0];
			for (i = 0; i < len; i++)
				printf("0x%x: 0x%x\n", reg+i, buff[i]);
		
		}
		else
		{
			printf("wrote 0x%x at 0x%x, len: %d \n", rdMem[0], reg, len );
		}
	}
	FT_Close(ftHandle);

exit:
	return 0;
}